//*****************************************************************************
//
// udma_adc_test.c - parameterized uDMA/GPIO test for validation
//
// Copyright (c) 2011 Texas Instruments Incorporated.  All rights reserved.
//
//*****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_ints.h"
#include "inc/hw_adc.h"
#include "driverlib/adc.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/udma.h"
#include "utils/uartstdio.h"

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>uDMA (udma_adc_test)</h1>
//!
//! This test application showing use of the ADC with uDMA.  The test is pretty
//! simple.  It performs several ADC acqusitions and verifies that the DMA
//! performed the expected transfers correctly.
//!
//! UART0, connected to the FTDI virtual COM port and running at 115,200,
//! 8-N-1, is used to display messages from this application.
//
//*****************************************************************************

//*****************************************************************************
//
// The size of the ADC buffer.
//
//*****************************************************************************
#define ADC_DSTBUF_SIZE        256

//*****************************************************************************
//
// Buffer used to store ADC samples.
//
//*****************************************************************************
static unsigned long g_ulAdcBuf[ADC_DSTBUF_SIZE];

//*****************************************************************************
//
// The count of uDMA errors.  This value is incremented by the uDMA error
// handler.
//
//*****************************************************************************
static volatile unsigned long g_uluDMAErrCount = 0;

//*****************************************************************************
//
// The count of times various error conditions were detected.
//
//*****************************************************************************
static volatile unsigned long g_ulBadPeriphIsr1;
static volatile unsigned long g_ulBadPeriphIsr2;

//*****************************************************************************
//
// Counter keeping track of ADC interrupts.
//
//*****************************************************************************
static volatile unsigned long g_ulIntCount;

//*****************************************************************************
//
// The control table used by the uDMA controller.  This table must be aligned
// to a 1024 byte boundary.
//
//*****************************************************************************
#if defined(ewarm)
#pragma data_alignment=1024
tDMAControlTable sDMAControlTable[64];
#elif defined(ccs)
#pragma DATA_ALIGN(sDMAControlTable, 1024)
tDMAControlTable sDMAControlTable[64];
#else
tDMAControlTable sDMAControlTable[64] __attribute__ ((aligned(1024)));
#endif

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
    UARTprintf("DriverLib ERROR in %s:%u\n", pcFilename, ulLine);
    for(;;)
    {
    }
}
#endif

//*****************************************************************************
//
// The interrupt handler for uDMA errors.  This interrupt will occur if the
// uDMA encounters a bus error while trying to perform a transfer.  This
// handler just increments a counter if an error occurs.
//
//*****************************************************************************
void
uDMAErrorHandler(void)
{
    unsigned long ulStatus;

    //
    // Check for uDMA error bit
    //
    ulStatus = MAP_uDMAErrorStatusGet();

    //
    // If there is a uDMA error, then clear the error and increment
    // the error counter.
    //
    if(ulStatus)
    {
        MAP_uDMAErrorStatusClear();
        g_uluDMAErrCount++;
    }
}

//*****************************************************************************
//
// The interrupt handler for ADC0, sequence 0.  This interrupt will occur when
// a DMA transfer is complete using the first sequence on ADC0.  It will also
// be triggered if the peripheral signals an error.
//
//*****************************************************************************
void
ADC0IntHandler(void)
{
    unsigned long ulStatus;

    //
    // Clear the ADC interrupt
    //
    MAP_ADCIntClear(ADC0_BASE, 0);

    //
    // Check to see if channel is done (it should be if we get the interrupt)
    //
    if(!MAP_uDMAChannelIsEnabled(UDMA_CHANNEL_ADC0))
    {
        //
        // Verify count remaining is 0. Note that this does not call ROM since
        // the ROM-based function appears to have a bug in earlier versions
        // of the 9B96 silicon and returns the value 1 rather than 0 when the
        // channel is stopped.
        //
        ulStatus = uDMAChannelSizeGet(UDMA_CHANNEL_ADC0);
        if(ulStatus)
        {
            g_ulBadPeriphIsr1++;
            return;
        }

        //
        // Increment ISR counter
        //
        g_ulIntCount++;

        //
        // Since this transfer is now done, disable the interrupt so the
        // handler is not called any more until set up again.  Also need
        // to unpend in case it happened again since we entered this
        // handler
        //
        MAP_ADCIntDisable(ADC0_BASE, 0);
        MAP_IntPendClear(INT_ADC0SS0);
    }

    //
    // Otherwise there is an error
    //
    else
    {
        g_ulBadPeriphIsr2++;
    }
}

//*****************************************************************************
//
// Initialize the peripheral DMA
//
//*****************************************************************************
void
InitADC0Transfer(void)
{
    unsigned int uIdx;

    //
    // Init buffers
    //
    for(uIdx = 0; uIdx < ADC_DSTBUF_SIZE; uIdx++)
    {
        g_ulAdcBuf[uIdx] = 0;
    }

    //
    // Configure peripheral
    //
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0);
    MAP_SysCtlPeripheralReset(SYSCTL_PERIPH_ADC0);
    MAP_ADCSequenceConfigure(ADC0_BASE, 0, ADC_TRIGGER_PROCESSOR, 0);
    MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 0, ADC_CTL_CH10 | ADC_CTL_IE);
    
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 1, ADC_CTL_CH2);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 2, ADC_CTL_CH3);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 3, ADC_CTL_CH4 | ADC_CTL_IE);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 4, ADC_CTL_CH5);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 5, ADC_CTL_CH6);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 6, ADC_CTL_CH7);
 //   MAP_ADCSequenceStepConfigure(ADC0_BASE, 0, 7, ADC_CTL_CH8 | ADC_CTL_IE |
 //                                ADC_CTL_END);

    //
    // Enable the sequencer
    //
    MAP_ADCSequenceEnable(ADC0_BASE, 0);

    //
    // Enable sequencer interrupts
    //
    MAP_ADCIntEnable(ADC0_BASE, 0);

    //
    // Configure attributes for DMA channel
    //
    MAP_uDMAChannelAttributeDisable(UDMA_CHANNEL_ADC0,
                                    UDMA_ATTR_ALTSELECT | UDMA_ATTR_USEBURST |
                                    UDMA_ATTR_HIGH_PRIORITY |
                                    UDMA_ATTR_REQMASK);

    //
    // Configure control parameters for DMA channel
    //
    MAP_uDMAChannelControlSet(UDMA_CHANNEL_ADC0 | UDMA_PRI_SELECT,
                              UDMA_SIZE_32 | UDMA_SRC_INC_NONE |
                              UDMA_DST_INC_32 | UDMA_ARB_4);

    //
    // Set up the transfer for the DMA channel
    //
    MAP_uDMAChannelTransferSet(UDMA_CHANNEL_ADC0 | UDMA_PRI_SELECT,
                               UDMA_MODE_BASIC,
                               (void *)(ADC0_BASE + ADC_O_SSFIFO0 + (0x20 * 0)),
                               g_ulAdcBuf, ADC_DSTBUF_SIZE);

    //
    // Enable the DMA channel
    //
    MAP_uDMAChannelEnable(UDMA_CHANNEL_ADC0);
}

//*****************************************************************************
//
// Set ADC0, sequence 0 to trigger constantly.
//
//*****************************************************************************
void
ReconfigADCTrigger(void)
{
    MAP_ADCSequenceDisable(ADC0_BASE, 0);
    MAP_ADCSequenceConfigure(ADC0_BASE, 0, ADC_TRIGGER_ALWAYS, 0);
    MAP_ADCSequenceEnable(ADC0_BASE, 0);
}

//*****************************************************************************
//
// Tidy up before exiting the test.
//
//*****************************************************************************
void
CleanupADC(void)
{
    MAP_ADCSequenceConfigure(ADC0_BASE, 0, ADC_TRIGGER_ALWAYS, 0);
    MAP_ADCSequenceDisable(ADC0_BASE, 0);

    //
    // Unregister the int handler for this channel
    //
    MAP_ADCIntDisable(ADC0_BASE, 0);

    //
    // Turn off the DMA channels for this peripheral
    //
    MAP_uDMAChannelDisable(UDMA_CHANNEL_ADC0);
}

//*****************************************************************************
//
// Check the status of DMA.  Verifies that the specified number of items
// have been transferred, and that no other error flasg or counters are
// set
//
// Returns 0 if okay, non-zero if error
//
//*****************************************************************************
unsigned long
CheckDMAStatus(unsigned long ulCount)
{
    unsigned long ulErr = 0;
    unsigned long ulRemaining;
    unsigned long ulExpected;

    if(g_uluDMAErrCount)
    {
        UARTprintf("g_uluDMAErrCount=%d\n", g_uluDMAErrCount);
        ulErr = 1;
    }
    if(g_ulBadPeriphIsr1 || g_ulBadPeriphIsr2)
    {
        UARTprintf("g_ulBadPeriphIsr1/2=%d/%d\n",
                   g_ulBadPeriphIsr1, g_ulBadPeriphIsr2);
        ulErr = 1;
    }
    if(g_ulIntCount && (ulCount != ADC_DSTBUF_SIZE))
    {
        UARTprintf("g_ulIntCount=%d\n", g_ulIntCount);
        ulErr = 1;
    }
    else if((g_ulIntCount != 1) && (ulCount == ADC_DSTBUF_SIZE))
    {
        UARTprintf("g_ulIntCount=%d, expected 1\n", g_ulIntCount);
        ulErr = 1;
    }

    //
    // How much data is still to be transfered? Note that this does not call
    // the ROM since the ROM-based function appears to have a bug in earlier
    // versions of the 9B96 silicon and returns the value 1 rather than 0 when
    // the channel is stopped.
    //
    ulRemaining =  uDMAChannelSizeGet(UDMA_CHANNEL_ADC0);
    ulExpected = ADC_DSTBUF_SIZE - ulCount;
    if(ulRemaining != ulExpected)
    {
        UARTprintf("uDMA channel %d has wrong count: expected %d, found %d\n",
                UDMA_CHANNEL_ADC0 & 0xff, ulExpected, ulRemaining);
        ulErr = 1;
    }

    if(ulErr)
    {
        UARTprintf("ERROR: DMA channel verification failed\n");
    }
    return(ulErr);
}

//*****************************************************************************
//
// Trigger the ADC a given number of times.
//
//*****************************************************************************
void
PulseADCReq(unsigned long ulCount)
{
    UARTprintf("... %d pulses\n", ulCount);
    while(ulCount--)
    {
        MAP_ADCProcessorTrigger(ADC0_BASE, 0);
        MAP_SysCtlDelay(5000);
    }
}

//*****************************************************************************
//
// This example demonstrates how to use the uDMA controller to transfer data
// between memory buffers and to and from a peripheral, in this case a UART.
// The uDMA controller is configured to repeatedly transfer a block of data
// from one memory buffer to another.  It is also set up to repeatedly copy a
// block of data from a buffer to the UART output.  The UART data is looped
// back so the same data is received, and the uDMA controlled is configured to
// continuously receive the UART data using ping-pong buffers.
//
// The processor is put to sleep when it is not doing anything, and this allows
// collection of CPU usage data to see how much CPU is being used while the
// data transfers are ongoing.
//
//*****************************************************************************
int
main(void)
{
    unsigned long ulErrExit = 0,i;

    //
    // Set the clocking to run from the xtal at 6 MHz.
    //
    SysCtlClockSet(SYSCTL_SYSDIV_1 | SYSCTL_USE_OSC | SYSCTL_OSC_MAIN |
                       SYSCTL_XTAL_16MHZ);
   
   
   // Enable the ADC hardware
   MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0);

   // Configure the pin as analog input
   MAP_SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOB ); // PB
   MAP_GPIOPinTypeADC( GPIO_PORTB_BASE, GPIO_PIN_4 ); // PB4

    //
    // Initialize the UART for console.
    //
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    MAP_SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_UART0);
    MAP_GPIOPinConfigure(GPIO_PA0_U0RX);
    MAP_GPIOPinConfigure(GPIO_PA1_U0TX);
    MAP_GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    UARTprintf("\nuDMA/ADC Test\n");
    UARTprintf(  "-------------\n");

    //
    // Show the clock frequency
    //
    UARTprintf("Stellaris @ %u MHz\n\n", SysCtlClockGet() / 1000000);

    //
    // Enable the uDMA controller at the system level.
    //
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_UDMA);

    //
    // Enable the uDMA controller error interrupt.  This interrupt will occur
    // if there is a bus error during a transfer.
    //
    MAP_IntEnable(INT_UDMAERR);
   
    //
    // Enable the uDMA controller.
    //
    MAP_uDMAEnable();

    //
    // Point at the control table to use for channel control structures.
    //
    MAP_uDMAControlBaseSet(sDMAControlTable);

    UARTprintf("Testing ADC0 seq 0 on DMA channel %d\n",
               UDMA_CHANNEL_ADC0 & 0xff);

    //
    // Initialize the ADC transfers
    //
    ulErrExit = 0;
    g_ulIntCount = 0;
    InitADC0Transfer();
    MAP_IntEnable(INT_ADC0);

    //
    // Trigger the ADC once and verify that we read 8 samples.
    //
    PulseADCReq(3);
    if(CheckDMAStatus(8))
    {
        ulErrExit = 1;
    }

    
    for (i=0;i<ADC_DSTBUF_SIZE;i++) {
       UARTprintf(" [%d] = %d\n",i,g_ulAdcBuf[i]);
    }
    //
    // Trigger the ADC 20 more times and verify that we read the expected
    // number of samples.
    //
//    PulseADCReq(20);
//    if(CheckDMAStatus(21 * 8))
//    {
//        ulErrExit = 1;
//    }

    //
    // Turn on continuous trigger so it will finish the DMA transfer.
    //
    ReconfigADCTrigger();

    //
    // Wait for it to finish.
    //
    while(!g_ulIntCount)
    {
    }

    if(CheckDMAStatus(256))
    {
        ulErrExit = 1;
    }

    //
    // Tidy up now that we are finished.
    //
    CleanupADC();


    //
    // If the last test had any errors, then just break out of test case
    // loop and run no more test cases
    //
    if(ulErrExit)
    {
        UARTprintf("Test ended with errors\n");
        while(1)
        {
        }
    }

    //
    // Reaching this point means the test case was successful so print a
    // completion message.
    //
    UARTprintf("TEST COMPLETE - OK\n");

    while(1)
    {
    }
}