//*****************************************************************************
//
// udma_adc_pin.c - ADC pin triggered acquisition with DMA transfer
//
// Copyright (c) 2009 Texas Instruments Incorporated.  All rights reserved.
// TI Information - Selective Disclosure
// 
// Texas Instruments Inc. (TI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by TI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is an unreleased example.//
//*****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_udma.h"
#include "inc/hw_gpio.h"
#include "inc/hw_adc.h"
#include "inc/hw_ints.h"
#include "driverlib/sysctl.h"
#include "driverlib/adc.h"
#include "driverlib/gpio.h"
#include "driverlib/udma.h"
#include "driverlib/pwm.h"
#include "driverlib/interrupt.h"
#include "utils/uartstdio.h"

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>ADC pin-triggered acquisition with uDMA (udma_adc_pin)</h1>
//!
//! This example demonstrates using an external pin to trigger an ADC
//! acquisition, and using the uDMA controller to transfer the ADC samples
//! into a buffer.
//!
//! The pin used for triggering the ADC is GPIO PB4.  This is the pin that must
//! be used for this feature.  This example sets up a PWM output on pin PC4 for
//! the purpose of simulating the external trigger.  In order to run the
//! example, pin PC4 must be jumpered to pin PB4.
//!
//! The example will collect 1024 samples into a buffer, and then stop.
//!
//! UART0, connected to the FTDI virtual COM port and running at 115,200,
//! 8-N-1, is used to display messages from this application.
//
//*****************************************************************************

//*****************************************************************************
//
// The period of the PWM output signal which is used to trigger the ADC samples.
// This is the number of CPU cycles between rising edges on the pin.
//
//*****************************************************************************
#define TIMEOUT_VAL 100

//*****************************************************************************
//
// The buffer used for collecting ADC samples.
//
//*****************************************************************************
#define ADC_BUFFER_SIZE 1024
static unsigned short g_usADCBuf[ADC_BUFFER_SIZE];

//*****************************************************************************
//
// Counters used to count how many times certain ISR event occur.
//
//*****************************************************************************
static unsigned long g_ulDMAIntCount = 0;
static unsigned long g_ulBadISR = 0;
static unsigned long g_uluDMAErrCount = 0;

//*****************************************************************************
//
// The control table used by the uDMA controller.  This table must be aligned
// to a 1024 byte boundary.
//
//*****************************************************************************
#if defined(ewarm)
#pragma data_alignment=1024
tDMAControlTable sControlTable[64];
#elif defined(ccs)
#pragma DATA_ALIGN(sControlTable, 1024)
tDMAControlTable sControlTable[64];
#else
tDMAControlTable sControlTable[64] __attribute__ ((aligned(1024)));
#endif


//*****************************************************************************
//
// The value to be written to clear the GPIO interrupt pin for B4
//
//*****************************************************************************
static unsigned long g_ulGPIOIntClearMask = GPIO_PIN_4;

//*****************************************************************************
//
// The value to be written to perform a software request on the DMA channel
//
//*****************************************************************************
static unsigned long g_ulDMAReqMask = (1 << UDMA_CHANNEL_SW);

//*****************************************************************************
//
// Declaration of preload structure, referred to by the task list.
// It will be defined later.
//
//*****************************************************************************
tDMAControlTable g_sTaskListPreload;

//*****************************************************************************
//
// This is the task list that defines the DMA scatter-gather operation.
// Each "task" in the task list gets copied one at a time into the alternate
// control structure for the channel, where it is then executed.  Each time
// the DMA channel is triggered, it executes these tasks.
//
// Task 1: clear GPIO interrupt.
// Because the GPIO module uses the interrupt signal to trigger the ADC
// acquisition, the GPIO interrupt is set after the external pin event occurs.
// This interrupt must be cleared in order to be able to trigger any more
// ADC acquisitions.  Therefore, task 1 is used to clear the GPIO interrupt.
// It does this by copying 1 word from a memory location to the GPIO interrupt
// clear register.  The value that is copied is the interrupt mask value for
// the interrupt to be cleared, pin 4 in this case (pin B4).
//
// Task 2: request DMA transfer for ADC
// A separate DMA channel is set up to transfer words from the ADC FIFO to a
// buffer in memory.  This other channel is set up to transfer one word for
// each request.  So this task, task 2, performs a request on the other DMA
// channel by copying a word from a memory location to the DMA software request
// register.  The value that is copied is the bit mask for the other DMA
// channel.  So this task will cause the other DMA channel to be triggered.
//
// Task 3: reload the task list
// At the end of the task list, the DMA channel will stop running and become
// disabled.  In this example we want to be able to continue to trigger the
// task list to run for each external trigger.  To do this, the original DMA
// control structure entry for this channel is reloaded from a preloaded
// structure in memory.  However, the uDMA control only actually alters the
// control word, so only the control word needs to be copied from the preloaded
// value into the control table.  This task simply copies the control word from
// the preloaded structure to the control word in the DMA channel table.  So
// this task reloads the channel so that it is ready to run again (at the next
// request).
//
// A note about memory scatter-gather vs peripheral scatter-gather.  Peripheral
// scatter-gather will perform one task per request from the peripheral, whereas
// memory s-g will perform all the tasks and then stop.  Here we are using a
// hybrid approach.  We want all the tasks to run, so they are set for memory
// s-g.  However, after the channel structure is reloaded by task 3, we dont
// want it to continue to run.  We want it to stop and wait for the next
// peripheral request.  For this reason, the last task is set up as peripheral
// s-g.
//
// The DMA channel performing the scatter-gather operations is the ADC channel.
// This channel is triggered by the ADC.  So when the ADC acquisition occurs,
// it will trigger these operations.
//
// The other DMA channel used is the "software" channel.  This is treated as
// a peripheral-memory move with one transfer occurring with each request on
// this channel.  And that request occurs because of the scatter-gather ops
// on the ADC DMA channel.

//*****************************************************************************
tDMAControlTable g_TaskTable[] =
{
    //
    // Task 1: clear GPIO interrupt for pin B4.
    // Copy GPIO int mask value to GPIO int clear register
    //
    {
        //
        // Source: variable holding int mask value
        // Dest: GPIO int clear register
        //
        &g_ulGPIOIntClearMask,
        (void *)(GPIO_PORTB_BASE + GPIO_O_ICR),

        //
        // source: 32-bit size, no increment
        // dest: 32-bit size, no increment
        //
        UDMA_CHCTL_SRCINC_NONE | UDMA_CHCTL_SRCSIZE_32 |
        UDMA_CHCTL_DSTINC_NONE | UDMA_CHCTL_DSTSIZE_32 |
        UDMA_CHCTL_ARBSIZE_1 |                  // arb size is 1 (1 at a time)
        ((1 - 1) << UDMA_CHCTL_XFERSIZE_S) |     // count of items to xfer is 1
        UDMA_CHCTL_XFERMODE_MEM_SGA             // memory scatter-gather mode
    },

    //
    // Task 2: initiate request on "software" DMA channel
    // Copy channel request mask to DMA request register
    // This will cause another DMA transfer to start on a different channel
    //
    {
        //
        // Source: variable holding DMA request mask value for sw channel
        // Dest: DMA request register
        //
        &g_ulDMAReqMask,
        (void *)UDMA_SWREQ,

        //
        // source: 32-bit size, no increment
        // dest: 32-bit size, no increment
        //
        UDMA_CHCTL_SRCINC_NONE | UDMA_CHCTL_SRCSIZE_32 |
        UDMA_CHCTL_DSTINC_NONE | UDMA_CHCTL_DSTSIZE_32 |
        UDMA_CHCTL_ARBSIZE_1 |                  // arb size is 1 (1 at a time)
        ((1 - 1) << UDMA_CHCTL_XFERSIZE_S) |     // count of items to xfer is 1
        UDMA_CHCTL_XFERMODE_MEM_SGA             // memory scatter-gather mode
    },

    //
    // Task 3: reset task list
    // Reprogram this DMA channel by reloading the primary structure with
    // the pointers needed to copy the task list for SG mode.
    // Only the control word actually needs to be reloaded.
    // This will allow another DMA transfer to start on this channel the
    // next time a peripheral request occurs.
    //
    {
        //
        // Source: pointer to preinitialized channel control word
        // Dest: DMA ADC0 channel control word in the primary table entry
        // This will copy 1 words from the preloaded structure to the DMA table
        //
        &(g_sTaskListPreload.ulControl),
        &(sControlTable[UDMA_CHANNEL_ADC0].ulControl),

        //
        // source: 32-bit size, 32-bit increment
        // dest: 32-bit size, 32-bit increment
        //
        UDMA_CHCTL_SRCINC_32 | UDMA_CHCTL_SRCSIZE_32 |
        UDMA_CHCTL_DSTINC_32 | UDMA_CHCTL_DSTSIZE_32 |
        UDMA_CHCTL_ARBSIZE_1 |                  // arb 1
        ((1 - 1) << UDMA_CHCTL_XFERSIZE_S) |    // count of items to xfer is 1
        UDMA_CHCTL_XFERMODE_PER_SGA             // periph scatter gather mode
                                                // used so transfer will stop
                                                // and wait for next ADC req
    }
};

//*****************************************************************************
//
// This is the task list that defines the DMA scatter-gather operation.
// Each "task" in the task list gets copied one at a time into the alternate
// control structure for the channel, where it is then executed.  Each time
// the DMA channel is triggered, it executes these tasks.
//
// This is the preloaded channel control structure for the ADC channel.  The
// values in this structure are configured to perform a memory scatter-gather
// operation whenever a request is received.  This structure is copied over
// into the DMA control table by software at the beginning of this app.  After
// that, it is reloaded into the control table by a scatter-gather operation.
//
//*****************************************************************************
tDMAControlTable g_sTaskListPreload =
{
    //
    // Source of the scatter gather transfer is the start of the task list.
    // Note that it must point to the last location to copy.
    //
    &(g_TaskTable[2].ulSpare),

    //
    // Destination is the alternate structure for the ADC channel
    //
    &(sControlTable[UDMA_CHANNEL_ADC0 | UDMA_ALT_SELECT].ulSpare),

    //
    // Setup transfer parameters for memory scatter-gather,
    // transfer is 3 entried of 4 32 bit words, for a total of 12 transfers
    // The arb size is set to 4 so that 4 items are copied each time.
    // Each structure is 4 words.
    //
    UDMA_CHCTL_DSTINC_32 | UDMA_CHCTL_DSTSIZE_32 |  // src is 32 bit words
    UDMA_CHCTL_SRCINC_32 | UDMA_CHCTL_SRCSIZE_32 |  // dest is 32 bit words
    UDMA_CHCTL_ARBSIZE_4 |                          // arb size is 4
    (((3 * 4) - 1) << UDMA_CHCTL_XFERSIZE_S) |      // transfer size is 12
    UDMA_CHCTL_XFERMODE_PER_SG                      // memory scatter-gather
};

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}
#endif

//*****************************************************************************
//
// This function loads the channel control structure with the starting values,
// and then enables the DMA channel.
//
//*****************************************************************************
void
ConfigSGTaskList(void)
{
    //
    // Copy the value of the preload task list into the primary structure
    // to initialize it for the first time.  After the DMA starts it will be
    // re-initialized each time by the third s-g task.
    //
    sControlTable[UDMA_CHANNEL_ADC0].pvDstEndAddr =
                  g_sTaskListPreload.pvDstEndAddr;
    sControlTable[UDMA_CHANNEL_ADC0].pvSrcEndAddr =
                  g_sTaskListPreload.pvSrcEndAddr;
    sControlTable[UDMA_CHANNEL_ADC0].ulControl =
                  g_sTaskListPreload.ulControl;

    //
    // Enable the ADC DMA channel.  This will allow it to start running when
    // it receives a request from the peripheral
    //
    uDMAChannelEnable(UDMA_CHANNEL_ADC0);

}

//*****************************************************************************
//
// This function configures the "software" DMA channel to transfer 1024
// items from the ADC FIFO to a memory buffer.  It is configured to transfer
// 1 item at a time (arb size 1).  Basic mode is used which means a request
// must be received for each item to transfer.  The request is made by the
// second s-g task.
//
//*****************************************************************************
void
ConfigDMAMemoryChannel(void)
{
    //
    // Configure the SW channel to perform transfers from the ADC FIFO to
    // a memory buffer.  It will transfer 1 at a time (1 per request).
    // Requests are being made by the other DMA channel scatter-gather task.
    // The source increment is none since it is reading from the ADC FIFO
    // register.  The dest inc is 32 since it will advance through the memory
    // buffer as ADC samples are written.
    //
    uDMAChannelControlSet(UDMA_CHANNEL_SW | UDMA_PRI_SELECT,
                          UDMA_SIZE_16 | UDMA_SRC_INC_NONE |
                          UDMA_DST_INC_16 | UDMA_ARB_1);

    //
    // Set up the transfer parameters for the channel.  This will
    // configure the transfer buffers and the transfer size.  Basic mode is
    // used which means that a request is needed for each transfer.
    // The source is the ADC FIFO and the destination is the memory buffer
    // where the ADC samples are stored.
    //
    uDMAChannelTransferSet(UDMA_CHANNEL_SW | UDMA_PRI_SELECT,
                           UDMA_MODE_BASIC,
                           (void *)(ADC0_BASE + ADC_O_SSFIFO0),
                           g_usADCBuf, ADC_BUFFER_SIZE);

    //
    // Now the software channel is ready to start a transfer.  The channel
    // must be enabled.  Once enabled, for each request it will make one
    // transfer.  The requests are made by a scatter-gather task.
    //
    uDMAChannelEnable(UDMA_CHANNEL_SW);
}

//*****************************************************************************
//
// This function configures the ADC to collect one sample, triggered from
// external pin.
//
//*****************************************************************************
void
ConfigADC(void)
{
    //
    // Configure GPIO PB4 to trigger on rising edges.  We configure the
    // interrupt on the GPIO which allows it to make a request to the ADC,
    // but the interrupt is not enabled on the processor so the processor
    // will never actually service any GPIO interrupts
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_GPIOB);
    GPIOPinTypeGPIOInput(GPIO_PORTB_BASE, GPIO_PIN_4);
    GPIOIntTypeSet(GPIO_PORTB_BASE, GPIO_PIN_4, GPIO_RISING_EDGE);
    GPIOPinIntEnable(GPIO_PORTB_BASE, GPIO_PIN_4);
    GPIOPinIntClear(GPIO_PORTB_BASE, GPIO_PIN_4);

    //
    // Enable the ADC peripheral and the GPIO port for the ADC input pin.
    // Then configure the ADC pin for channel 0.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_ADC0);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_GPIOE);
    GPIOPinTypeADC(GPIO_PORTE_BASE, GPIO_PIN_7);

    //
    // Configure the ADC to take a sample based on external pin event.
    // Use sequencer 0 and set it to take one sample and assert interrupt.
    // The interrupt is used to trigger the DMA request, the interrupt is
    // not actually enabled to the processor.
    //
    ADCSequenceConfigure(ADC0_BASE, 0, ADC_TRIGGER_EXTERNAL, 0);
    ADCSequenceStepConfigure(ADC0_BASE, 0, 0,
                             ADC_CTL_CH0 | ADC_CTL_IE | ADC_CTL_END);
    ADCSequenceEnable(ADC0_BASE, 0);
}

//*****************************************************************************
//
// Set up the PWM0 output to be used as a signal source for the PC4 input
// pin.  This example application uses PWM0 as the signal source in order
// to demonstrate the ADC trigger feature.  In a real application some external
// signal would be used as the signal source for ADC trigger input.
//
//*****************************************************************************
static void
SetupSignalSource(void)
{
    //
    // Enable the GPIO port used for PWM0 output.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_GPIOC);

    //
    // Enable the PWM peripheral and configure the GPIO pin
    // used for PWM0.  GPIO pin D0 is used for PWM0 output.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_PWM);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_PWM);
    GPIOPinConfigure(GPIO_PC4_PWM6);
    GPIOPinTypePWM(GPIO_PORTC_BASE, GPIO_PIN_4);

    //
    // Configure the PWM0 output to generate a 50% square wave with a
    // period of TIMEOUT_VAL processor cycles.
    //
    PWMGenConfigure(PWM_BASE, PWM_GEN_3, PWM_GEN_MODE_DOWN);
    PWMGenPeriodSet(PWM_BASE, PWM_GEN_3, TIMEOUT_VAL);
    PWMPulseWidthSet(PWM_BASE, PWM_OUT_6, TIMEOUT_VAL / 2);
    PWMOutputState(PWM_BASE, PWM_OUT_6_BIT, 1);
    PWMGenEnable(PWM_BASE, PWM_GEN_3);
}

//*****************************************************************************
//
// The interrupt handler for uDMA errors.  This interrupt will occur if the
// uDMA encounters a bus error while trying to perform a transfer.  This
// handler just increments a counter if an error occurs.
//
//*****************************************************************************
void
uDMAErrorHandler(void)
{
    unsigned long ulStatus;

    //
    // Check for uDMA error bit
    //
    ulStatus = uDMAErrorStatusGet();

    //
    // If there is a uDMA error, then clear the error and increment
    // the error counter.
    //
    if(ulStatus)
    {
        uDMAErrorStatusClear();
        g_uluDMAErrCount++;
    }
}

//*****************************************************************************
//
// The interrupt handler for uDMA interrupts from the memory channel.  This
// handler will verify that transfer is complete, increment a counter, and
// disable the ADC DMA channel which is still enabled.
// In the real application this is likely where you would set a flag to
// indicate that the transfer is complete and that the data needs to be
// processed.  Another transfer could be set up and enabled here too.
//
//*****************************************************************************
void
uDMAIntHandler(void)
{
    unsigned long ulMode;

    //
    // Check for the primary control structure to indicate complete.
    //
    ulMode = uDMAChannelModeGet(UDMA_CHANNEL_SW);
    if(ulMode == UDMA_MODE_STOP)
    {
        //
        // Increment the count of completed transfers.
        //
        g_ulDMAIntCount++;
    }

    //
    // If the channel is not stopped, then something is wrong.
    //
    else
    {
        g_ulBadISR++;
    }

    //
    // Turn off the ADC DMA channel which is still enabled and will continue
    // to trigger s-g operations for each input pin edge.
    //
    uDMAChannelDisable(UDMA_CHANNEL_ADC0);
}

//*****************************************************************************
//
// The main function sets up the peripherals for the example, then enters
// a wait loop until the DMA transfers are complete.  At the end some
// information is printed for the user.
// Note that all peripherals are enabled for sleep mode operation since the
// processor is put to sleep while waiting for the transfers to complete.
//
//*****************************************************************************
int
main(void)
{
    unsigned long ulRemaining;
    unsigned long ulIdx;
    unsigned long ulCol;

    //
    // Set the clocking to run directly from the PLL.  Run at 50 MHz to
    // provide plenty of bus cycles for the DMA operations.
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_16MHZ);

    //
    // Initialize the UART and write a message to the terminal.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    UARTprintf("\033[2JPin Triggered ADC with uDMA\n\n");
    
   //                    
   // on active le convertisseur +5v  PBA6 = 1 pour alimenter la camera
   //
   GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, (GPIO_PIN_6) );
   GPIOPadConfigSet( GPIO_PORTA_BASE, GPIO_PIN_6, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD );
   GPIOPinWrite( GPIO_PORTA_BASE,(GPIO_PIN_6), 0x40);
 
    //
    // Configure the ADC.  This will set up the ADC to take one sample every
    // event on GPIO pin B4.
    //
    ConfigADC();

    //
    // Enable the uDMA peripheral.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UDMA);
    SysCtlPeripheralSleepEnable(SYSCTL_PERIPH_UDMA);

    //
    // Enable the uDMA controller.
    //
    uDMAEnable();

    //
    // Point at the control table to use for channel control structures.
    //
    uDMAControlBaseSet(sControlTable);

    //
    // This will configure the DMA "software" channel to transfer samples
    // from the ADC, once per request
    //
    ConfigDMAMemoryChannel();

    //
    // This will set up a scatter-gather task list that will be activated
    // each time the ADC signals an interrupt (the interrupt in this case is
    // used as a DMA request and does not interrupt the processor).
    // Once this is done, the system is "live".  Any transition on B4 will
    // cause the ADC to take one sample and transfer it to a memory buffer.
    //
    ConfigSGTaskList();

    //
    // Enable interrupts from the uDMA software channel.  This interrupt will
    // fire after all 1024 ADC samples have been transferred to a buffer.
    //
    IntEnable(INT_UDMA);

    //
    // Enable the uDMA controller error interrupt.  This interrupt will occur
    // if there is a bus error during a transfer.
    //
    IntEnable(INT_UDMAERR);

    //
    // Start the external ADC trigger signal.
    //
    SetupSignalSource();

    //
    // Put the processor to sleep to wait for the DMA transfers to complete.
    //
    UARTprintf("Procesor sleeping ... ");
    SysCtlSleep();

    //
    // Print a message that the processor has waked.
    //
    UARTprintf("Procesor wake on interrupt\n\n");

    //
    // Verify that the various interrupt counters are as expected.  The
    // DMA interrupt should have occurred one time, and the two error
    // indicating counters should be 0.
    //
    UARTprintf("uDMA completion interrupts = %d : %s\n", g_ulDMAIntCount,
               g_ulDMAIntCount == 1 ? "OK" : "ERROR");
    UARTprintf("uDMA spurious interrupts   = %d : %s\n", g_ulBadISR,
               g_ulBadISR == 0 ? "OK" : "ERROR");
    UARTprintf("uDMA spurious interrupts   = %d : %s\n", g_uluDMAErrCount,
               g_uluDMAErrCount == 0 ? "OK" : "ERROR");

    //
    // Make sure that there are no remaining items to transfer
    // Allow for a return value of 0 or 1 due to bug in function.
    //
    ulRemaining = uDMAChannelSizeGet(UDMA_CHANNEL_SW);
    UARTprintf("remaining items to transfer= %d : %s\n", ulRemaining,
               ulRemaining <= 1 ? "OK" : "ERROR");

    //
    // Enter a loop to print the acquired samples
    //
    UARTprintf("\nADC samples\n===========\n");
    for(ulIdx = 0; ulIdx < ADC_BUFFER_SIZE; ulIdx += 8)
    {
        UARTprintf("[0x%04X] ", ulIdx);
        for(ulCol = 0; ulCol < 8; ulCol++)
        {
            UARTprintf(" %04X", g_usADCBuf[ulIdx + ulCol]);
        }
        UARTprintf("\n");
    }

    //
    // End of program, loop forever
    //
    for(;;)
    {
    }

    return(0);
}
